@@ -10,7 +10,6 @@ import GHC.Debugger.Breakpoint
 import GHC.Debugger.Evaluation
 import GHC.Debugger.Stopped
 import GHC.Debugger.Monad
-import GHC.Debugger.Utils
 import GHC.Debugger.Interface.Messages
 import GHC.Debugger.Logger
 
@@ -23,19 +22,10 @@ execute :: Recorder (WithSeverity DebuggerLog) -> Command -> Debugger Response
 execute recorder = \case
   ClearFunctionBreakpoints -> DidClearBreakpoints <$ clearBreakpoints Nothing
   ClearModBreakpoints fp -> DidClearBreakpoints <$ clearBreakpoints (Just fp)
-  SetBreakpoint bp -> DidSetBreakpoint <$> setBreakpoint bp BreakpointEnabled
+  SetBreakpoint{brk, hitCount, condition} ->
+    DidSetBreakpoint <$> setBreakpoint brk (condBreakEnableStatus hitCount condition)
   DelBreakpoint bp -> DidRemoveBreakpoint <$> setBreakpoint bp BreakpointDisabled
-  GetBreakpointsAt ModuleBreak{path, lineNum, columnNum} -> do
-    mmodl <- getModuleByPath path
-    case mmodl of
-      Left e -> do
-        displayWarnings [e]
-        return $ DidGetBreakpoints Nothing
-      Right modl -> do
-        mbfnd <- getBreakpointsAt modl lineNum columnNum
-        return $
-          DidGetBreakpoints (realSrcSpanToSourceSpan . snd <$> mbfnd)
-  GetBreakpointsAt _ -> error "unexpected getbreakpoints without ModuleBreak"
+  GetBreakpointsAt bp -> DidGetBreakpoints <$> getBreakpointsAt bp
   GetStacktrace -> GotStacktrace <$> getStacktrace
   GetScopes -> GotScopes <$> getScopes
   GetVariables kind -> GotVariables <$> getVariables kind
@@ -55,3 +45,4 @@ data DebuggerLog
 instance Pretty DebuggerLog where
   pretty = \ case
     EvalLog msg -> pretty msg
+
