@@ -14,6 +14,8 @@ module GHC.Debugger.Evaluation where
 import GHC.Utils.Outputable
 import Control.Monad.IO.Class
 import Control.Monad.Catch
+import Control.Monad.Reader
+import Data.IORef
 import qualified Data.List as List
 import Data.Maybe
 import System.FilePath
@@ -39,7 +41,8 @@ import GHC.Debugger.Stopped.Variables
 import GHC.Debugger.Monad
 import GHC.Debugger.Utils
 import GHC.Debugger.Interface.Messages
-import GHC.Debugger.Logger
+import GHC.Debugger.Logger as Logger
+import qualified GHC.Debugger.Breakpoint.Map as BM
 
 data EvalLog
   = LogEvalModule GHC.Module
@@ -55,7 +58,6 @@ instance Pretty EvalLog where
 -- | Run a program with debugging enabled
 debugExecution :: Recorder (WithSeverity EvalLog) -> FilePath -> EntryPoint -> [String] {-^ Args -} -> Debugger EvalResult
 debugExecution recorder entryFile entry args = do
-
   -- consider always using :trace like ghci-dap to always have a stacktrace?
   -- better solution could involve profiling stack traces or from IPE info?
   modSummaryOfEntryFile <- findUnitIdOfEntryFile entryFile
@@ -184,22 +186,58 @@ doEval expr = do
     Right ExecBreak{} -> continueToCompletion >>= handleExecResult
     Right r@ExecComplete{} -> handleExecResult r
 
+-- | Resume execution with single step mode 'RunToCompletion', skipping all breakpoints we hit, until we reach 'ExecComplete'.
+--
+-- We use this in 'doEval' because we want to ignore breakpoints in expressions given at the prompt.
+continueToCompletion :: Debugger GHC.ExecResult
+continueToCompletion = do
+  execr <- GHC.resumeExec GHC.RunToCompletion Nothing
+  case execr of
+    GHC.ExecBreak{} -> continueToCompletion
+    GHC.ExecComplete{} -> return execr
+
 -- | Turn a GHC's 'ExecResult' into an 'EvalResult' response
 handleExecResult :: GHC.ExecResult -> Debugger EvalResult
 handleExecResult = \case
     ExecComplete {execResult} -> do
       case execResult of
         Left e -> return (EvalException (show e) "SomeException")
-        Right [] -> return (EvalCompleted "" "") -- Evaluation completed without binding any result.
+        Right [] -> return (EvalCompleted "" "" NoVariables) -- Evaluation completed without binding any result.
         Right (n:_ns) -> inspectName n >>= \case
-          Just VarInfo{varValue, varType} -> return (EvalCompleted varValue varType)
+          Just VarInfo{varValue, varType, varRef} -> do
+            return (EvalCompleted varValue varType varRef)
           Nothing     -> liftIO $ fail "doEval failed"
     ExecBreak {breakNames = _, breakPointId = Nothing} ->
       -- Stopped at an exception
       -- TODO: force the exception to display string with Backtrace?
       return EvalStopped{breakId = Nothing}
-    ExecBreak {breakNames = _, breakPointId} ->
-      return EvalStopped{breakId = breakPointId}
+    ExecBreak {breakNames = _, breakPointId = Just bid} -> do
+      bm <- liftIO . readIORef =<< asks activeBreakpoints
+      case BM.lookup bid bm of
+        -- todo: BreakpointAfterCountCond is not handled yet.
+        Just (BreakpointWhenCond cond, _) -> do
+          let evalFailedMsg e = text $ "Evaluation of conditional breakpoint expression failed with " ++ e ++ "\nIgnoring..."
+          let resume = GHC.resumeExec GHC.RunToCompletion Nothing >>= handleExecResult
+          doEval cond >>= \case
+            EvalStopped{} -> error "impossible for doEval"
+            EvalCompleted { resultVal, resultType } ->
+              if resultType == "Bool" then do
+                if resultVal == "True" then
+                  return EvalStopped{breakId = Just bid}
+                else
+                  resume
+              else do
+                logSDoc Logger.Warning (evalFailedMsg "\"expression resultType is != Bool\"")
+                resume
+            EvalException { resultVal } -> do
+              logSDoc Logger.Warning (evalFailedMsg resultVal)
+              resume
+            EvalAbortedWith e -> do
+              logSDoc Logger.Warning (evalFailedMsg e)
+              resume
+
+        -- Unconditionally 'EvalStopped' in all other cases
+        _ -> return EvalStopped{breakId = Just bid}
 
 -- | Get the value and type of a given 'Name' as rendered strings in 'VarInfo'.
 inspectName :: Name -> Debugger (Maybe VarInfo)
