@@ -36,6 +36,7 @@ import qualified Data.Containers.ListUtils as L
 import GHC.ResponseFile (expandResponse)
 import HIE.Bios.Environment as HIE
 import System.FilePath
+import Data.Time
 import qualified System.Directory as Directory
 import qualified System.Environment as Env
 
@@ -52,6 +53,10 @@ import qualified GHC.Unit.State                        as State
 import GHC.Driver.Env
 import GHC.Types.SrcLoc
 import Language.Haskell.Syntax.Module.Name
+import qualified Data.Foldable as Foldable
+import qualified GHC.Unit.Home.Graph as HUG
+import Data.Maybe
+import GHC.Types.Target (InputFileBuffer)
 
 -- | Throws if package flags are unsatisfiable
 parseHomeUnitArguments :: GhcMonad m
@@ -89,7 +94,7 @@ parseHomeUnitArguments cfp compRoot units theOpts dflags rootDir = do
         -- Canonicalize! Why? Because the targets we get from the cradle are normalised and if we don't normalise the "special target" then they aren't deduplicated properly.
         canon_fp <- liftIO $ Directory.canonicalizePath abs_fp
         let special_target = mkSimpleTarget df canon_fp
-        pure $ (df, special_target : targets) NonEmpty.:| []
+        pure $ (df, if null targets then [special_target] else targets) NonEmpty.:| []
     where
       initMulti unitArgFiles =
         forM unitArgFiles $ \f -> do
@@ -157,13 +162,11 @@ initHomeUnitEnv unitDflags env = do
   -- additionally, set checked dflags so we don't lose fixes
   initial_home_graph <- createUnitEnvFromFlags dflags0 unitDflags
   let home_units = unitEnv_keys initial_home_graph
-  home_unit_graph <- forM initial_home_graph $ \homeUnitEnv -> do
+  init_home_unit_graph <- forM initial_home_graph $ \homeUnitEnv -> do
     let cached_unit_dbs = homeUnitEnv_unit_dbs homeUnitEnv
         dflags = homeUnitEnv_dflags homeUnitEnv
         old_hpt = homeUnitEnv_hpt homeUnitEnv
-
-    (dbs,unit_state,home_unit,mconstants) <- State.initUnits (hsc_logger env) dflags cached_unit_dbs home_units
-
+    (dbs,unit_state,home_unit,mconstants) <- State.initUnits (hsc_logger env) dflags Nothing home_units
     updated_dflags <- GHC.updatePlatformConstants dflags mconstants
     pure HomeUnitEnv
       { homeUnitEnv_units = unit_state
@@ -173,6 +176,24 @@ initHomeUnitEnv unitDflags env = do
       , homeUnitEnv_home_unit = Just home_unit
       }
 
+  let cached_unit_dbs = concat . catMaybes . fmap homeUnitEnv_unit_dbs $ Foldable.toList init_home_unit_graph
+
+  let homeUnitEnv = fromJust $ HUG.unitEnv_lookup_maybe interactiveGhcDebuggerUnitId init_home_unit_graph
+      dflags = homeUnitEnv_dflags homeUnitEnv
+      old_hpt = homeUnitEnv_hpt homeUnitEnv
+  (dbs,unit_state,home_unit,mconstants) <- State.initUnits (hsc_logger env) dflags (Just cached_unit_dbs) home_units
+
+  updated_dflags <- GHC.updatePlatformConstants dflags mconstants
+  let ie = HomeUnitEnv
+       { homeUnitEnv_units = unit_state
+       , homeUnitEnv_unit_dbs = Just dbs
+       , homeUnitEnv_dflags = updated_dflags
+       , homeUnitEnv_hpt = old_hpt
+       , homeUnitEnv_home_unit = Just home_unit
+       }
+
+  let home_unit_graph = HUG.unitEnv_insert interactiveGhcDebuggerUnitId ie init_home_unit_graph
+
   let dflags1 = homeUnitEnv_dflags $ unitEnv_lookup interactiveGhcDebuggerUnitId home_unit_graph
   let unit_env = UnitEnv
         { ue_platform        = targetPlatform dflags1
@@ -203,7 +224,7 @@ setupMultiHomeUnitGhcSession exts hsc_env cis = do
     ts <- forM cis $ \(df, targets) -> do
       -- evaluate $ liftRnf rwhnf targets
 
-      let mk t = fromTargetId (importPaths df) exts (homeUnitId_ df) (GHC.targetId t)
+      let mk t = fromTargetId (importPaths df) exts (homeUnitId_ df) (GHC.targetId t) (GHC.targetContents t)
       ctargets <- concatMapM mk targets
 
       return (L.nubOrdOn targetTarget ctargets)
@@ -220,8 +241,8 @@ data TargetDetails = TargetDetails
   -- convenient lookup table from 'FilePath' to 'TargetDetails'.
   , targetUnitId :: UnitId
   -- ^ UnitId of 'targetTarget'.
+  , targetContents :: Maybe (InputFileBuffer, UTCTime)
   }
-  deriving (Eq, Ord)
 
 -- | A simplified view on a 'TargetId'.
 --
@@ -231,29 +252,30 @@ data Target = TargetModule ModuleName | TargetFile FilePath
 
 -- | Turn a 'TargetDetails' into a 'GHC.Target'.
 toGhcTarget :: TargetDetails -> GHC.Target
-toGhcTarget (TargetDetails tid _ uid) = case tid of
-  TargetModule modl -> GHC.Target (GHC.TargetModule modl) True uid Nothing
-  TargetFile fp -> GHC.Target (GHC.TargetFile fp Nothing) True uid Nothing
+toGhcTarget (TargetDetails tid _ uid cts) = case tid of
+  TargetModule modl -> GHC.Target (GHC.TargetModule modl) True uid cts
+  TargetFile fp -> GHC.Target (GHC.TargetFile fp Nothing) True uid cts
 
 fromTargetId :: [FilePath]          -- ^ import paths
              -> [String]            -- ^ extensions to consider
              -> UnitId
              -> GHC.TargetId
+             -> Maybe (InputFileBuffer, UTCTime)
              -> IO [TargetDetails]
 -- For a target module we consider all the import paths
-fromTargetId is exts unitId (GHC.TargetModule modName) = do
+fromTargetId is exts unitId (GHC.TargetModule modName) ctts = do
     let fps = [i </> moduleNameSlashes modName -<.> ext <> boot
               | ext <- exts
               , i <- is
               , boot <- ["", "-boot"]
               ]
-    return [TargetDetails (TargetModule modName) fps unitId]
+    return [TargetDetails (TargetModule modName) fps unitId ctts]
 -- For a 'TargetFile' we consider all the possible module names
-fromTargetId _ _ unitId (GHC.TargetFile f _) = do
+fromTargetId _ _ unitId (GHC.TargetFile f _) ctts = do
     let other
           | "-boot" `L.isSuffixOf` f = dropEnd 5 f
           | otherwise = (f ++ "-boot")
-    return [TargetDetails (TargetFile f) [f, other] unitId]
+    return [TargetDetails (TargetFile f) [f, other] unitId ctts]
 
 -- ----------------------------------------------------------------------------
 -- GHC Utils that should likely be exposed by GHC
