@@ -1,13 +1,18 @@
 {-# LANGUAGE CPP, NamedFieldPuns, TupleSections, LambdaCase,
    DuplicateRecordFields, RecordWildCards, TupleSections, ViewPatterns,
    TypeApplications, ScopedTypeVariables, BangPatterns #-}
-module GHC.Debugger.Utils where
+module GHC.Debugger.Utils
+  ( module GHC.Debugger.Utils
+  , module GHC.Utils.Outputable
+  , module GHC.Utils.Trace
+  ) where
 
 import GHC
 import GHC.Data.FastString
-import GHC.Driver.DynFlags as GHC
-import GHC.Driver.Ppr as GHC
-import GHC.Utils.Outputable as GHC
+import GHC.Driver.DynFlags
+import GHC.Driver.Ppr
+import GHC.Utils.Outputable
+import GHC.Utils.Trace
 
 import GHC.Debugger.Monad
 import GHC.Debugger.Interface.Messages
