@@ -52,11 +52,13 @@ import Development.Debug.Session.Setup
 
 data InitLog
   = DebuggerLog Debugger.DebuggerLog
+  | DebuggerMonadLog Debugger.DebuggerMonadLog
   | FlagsLog FlagsLog
 
 instance Pretty InitLog where
   pretty = \ case
     DebuggerLog msg -> pretty msg
+    DebuggerMonadLog msg -> pretty msg
     FlagsLog msg -> pretty msg
 
 --------------------------------------------------------------------------------
@@ -125,7 +127,6 @@ initDebugger l supportsRunInTerminal
             , supportsANSIHyperlinks = False -- VSCode does not support this
             }
 
-
       -- Create pipes to read/write the debugger (not debuggee's) output.
       -- The write end is given to `runDebugger` and the read end is continuously
       -- read from until we read an EOF.
@@ -236,8 +237,6 @@ debuggerThread :: Recorder (WithSeverity InitLog)
                -> IO ()
 debuggerThread recorder finished_init writeDebuggerOutput workDir HieBiosFlags{..} extraGhcArgs mainFp runConf requests replies withAdaptor = do
 
-  let finalGhcInvocation = ghcInvocation ++ extraGhcArgs
-
   -- See Notes (CWD) above
   setCurrentDirectory workDir
 
@@ -246,13 +245,12 @@ debuggerThread recorder finished_init writeDebuggerOutput workDir HieBiosFlags{.
     Output.console $ T.pack $
       "libdir: " <> libdir <> "\n" <>
       "units: " <> unwords units <> "\n" <>
-      "args: " <> unwords finalGhcInvocation
+      "args: " <> unwords (ghcInvocation ++ extraGhcArgs)
 
   catches
     (do
-      Debugger.runDebugger writeDebuggerOutput rootDir componentDir libdir units finalGhcInvocation mainFp runConf $ do
+      Debugger.runDebugger (cmapWithSev DebuggerMonadLog recorder) writeDebuggerOutput rootDir componentDir libdir units ghcInvocation extraGhcArgs mainFp runConf $ do
         liftIO $ signalInitialized (Right ())
-
         forever $ do
           req <- takeMVar requests & liftIO
           resp <- (Debugger.execute (cmapWithSev DebuggerLog recorder) req <&> Right)
