@@ -27,12 +27,14 @@ type InteractiveDM a = InputT (RWST (FilePath{-entry file-},String{-entry point-
 
 data InteractiveLog
   = DebuggerLog DebuggerLog
+  | DebuggerMonadLog DebuggerMonadLog
   | FlagsLog FlagsLog
 
 instance Pretty InteractiveLog where
   pretty = \ case
     DebuggerLog msg -> pretty msg
     FlagsLog msg -> pretty msg
+    DebuggerMonadLog msg -> pretty msg
 
 -- | Run it
 runIDM :: Recorder (WithSeverity InteractiveLog)
@@ -58,10 +60,10 @@ runIDM logger entryPoint entryFile entryArgs extraGhcArgs act = do
             , supportsANSIHyperlinks = False
             }
 
-      let finalGhcInvocation = ghcInvocation ++ extraGhcArgs
       let absEntryFile = normalise $ projectRoot </> entryFile
+      let debugRec = cmapWithSev DebuggerMonadLog logger
 
-      runDebugger stdout rootDir componentDir libdir units finalGhcInvocation absEntryFile defaultRunConf $
+      runDebugger debugRec stdout rootDir componentDir libdir units ghcInvocation extraGhcArgs absEntryFile defaultRunConf $
         fmap fst $
           evalRWST (runInputT (setComplete noCompletion defaultSettings) act)
                    (entryFile, entryPoint, entryArgs) Nothing
@@ -164,6 +166,20 @@ breakpointParser =
   ( flag' OnUncaughtExceptionsBreak ( long "error" )
   )
 
+conditionalBreakParser :: Parser (Maybe String)
+conditionalBreakParser =
+  optional (option str
+    ( long "condition"
+   <> metavar "CONDITION"
+   <> help "Only stop when CONDITION evaluates to True" ))
+
+hitCountBreakParser :: Parser (Maybe Int)
+hitCountBreakParser =
+  optional (option auto
+    ( long "hit"
+   <> metavar "N:INT"
+   <> help "Ignore first N:INT times this breakpoint is hit" ))
+
 runParser :: FilePath -> String -> [String] -> Parser Command
 runParser entryFile entryPoint entryArgs =
   -- --entry <name> with some args
@@ -194,7 +210,7 @@ runParser entryFile entryPoint entryArgs =
 cmdParser :: FilePath -> String -> [String] -> Parser Command
 cmdParser entryFile entryPoint entryArgs = hsubparser
   ( Options.Applicative.command "break"
-    ( info (SetBreakpoint <$> breakpointParser)
+    ( info (SetBreakpoint <$> breakpointParser <*> hitCountBreakParser <*> conditionalBreakParser)
       ( progDesc "Set a breakpoint" ) )
   <>
     Options.Applicative.command "delete"
